/*
*  Rubrica
*  file: sort.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include "rubrica.h"
#include "sort.h"

Ordinamento ordinamento;

extern GList *rubrica;
extern VistaDati vista;

/*  ordinamento delle schede */

void rubrica_init_ordinamento(void)
{
  ordinamento.modo  = ALFABETICO;
  ordinamento.tipo  = CRESCENTE;
  ordinamento.campo = SCHEDA;
}

/* 
   alfa:
   Confronta i due elementi a e b
   
    ordinamento  | a == b | a < b | a > b | a == NULL | b == NULL |
   +-------------+------------------------+-----------+-----------+
   |crescente    |   0    |   -1  |   1   |    -1     |     1     |  valore ritornato
   +-------------+--------+-------+-------+-----------+-----------+
   |decrescente  |   0    |    1  |  -1   |     1     |    -1     |
   +-------------+--------+-------+-------+-----------+-----------+
*/


gint alfa(gconstpointer a, gconstpointer b)
{
  RubricaItem *p, *s;
  gchar *str1, *str2;

  p = (RubricaItem *)a;  /* prima */
  s = (RubricaItem *)b;  /* seconda */

  switch(ordinamento.campo)
    {
    case SCHEDA:
      str1 = p->scheda;
      str2= s->scheda;
      break;
    case NOME:
      str1 = p->nome;
      str2 = s->nome;      
      break;
    case COGNOME:
      str1 = p->cognome;
      str2 = s->cognome;
      break;
    default:
      str1 = p->scheda;
      str2 = s->scheda;
      break;
    }

  if (!str1 && !str2)
    return 0;

  if (ordinamento.tipo == CRESCENTE)
    {
      if (!str1)
	return -1;

      if (!str2)
	return 1;

      if (strcmp(str1,str2) < 0)
	return -1;
      if (strcmp(str1,str2) > 0)
	return 1;
    }

  if (ordinamento.tipo == DECRESCENTE)
    {
      if (!str1)
	return 1;
      
      if (!str2)
	return -1;
      
      if (strcmp(str1,str2) > 0)
	return -1;
      if (strcmp(str1,str2) < 0)
	return 1;
    }  

  /* a == b */
  return 0;
}

void rubrica_riordina_lista(void)
{
  gint n, i;
  GtkCList *cl;

  i=0;
  if(ordinamento.modo == ALFABETICO)
      rubrica = g_list_sort(rubrica, (GCompareFunc) alfa);

  /* TO DO:
     ordinamento per grandezza
     ordinamento per data
  */

  n = rubrica_vista_get_indirizzi();
  cl = rubrica_vista_get_schede();
  gtk_clist_clear(cl);

  for(i=0; i < n; i++)
    {
      gchar *buffer[5];
      RubricaItem *r;

      r = rubrica_prendi_elemento(i);

      buffer[0] = "";                    // cella per l'eventuale pixmap
      buffer[1] = g_strdup(r->scheda);
      buffer[2] = g_strdup(r->nome);
      buffer[3] = g_strdup(r->cognome);
      buffer[4] = NULL;
      
      gtk_clist_insert(GTK_CLIST(vista.schede), n, buffer);
    }
}

void  rubrica_ordinamento_set_campo(ORDINARE n)
{
  ordinamento.campo = (ORDINARE) n;
}


Ordinamento rubrica_get_ordinamento(void)
{
  return (Ordinamento) ordinamento;
}

void  rubrica_ordinamento_set_alfabetico()
{
  ordinamento.modo=ALFABETICO;	
}

void  rubrica_ordinamento_set_grandezza()
{
 /* TO DO */ 
}

void  rubrica_ordinamento_set_data()
{
 /* TO DO */ 

}

void  rubrica_ordinamento_set_crescente()
{
  ordinamento.tipo = CRESCENTE;
}

void  rubrica_ordinamento_set_decrescente()
{
  ordinamento.tipo = DECRESCENTE;
}

